//---------------------------------------------------------------
// longrunctl.c
//
// Simple command line Longrun controller application
// Anders Granlund
//---------------------------------------------------------------
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>
#include "longrun.h"

//---------------------------------------------------------------
// Longrun mode names
//---------------------------------------------------------------
char *longrun_names[LONGRUN_MODE_MAX + 1] =
{
	"minfrequency",
	"economy",
	"performance",
	"maxfrequency",
	"unknown",
};

//---------------------------------------------------------------
// Command line argument selections
//---------------------------------------------------------------
enum
{
	SELECTION_ERROR 		= -1,
	SELECTION_STATUS 		= 0,
	SELECTION_ECONOMY 		= 1,
	SELECTION_PERFORMANCE 	= 2,
	SELECTION_MINFREQUENCY	= 3,
	SELECTION_MAXFREQUENCY	= 4,
};

//---------------------------------------------------------------
// PrintInfo
//---------------------------------------------------------------
void PrintInfo()
{
	printf("longrunctl for Transmeta Crusoe BeOS LongRun Driver\n");
	printf("usage:\n");
	printf("\tlongrunctl status       - Print longrun status\n");
	printf("\tlongrunctl economy      - Set economy mode\n");
	printf("\tlongrunctl performance  - Set performance mode\n");
	printf("\tlongrunctl minfrequency - Set minimum frequency mode\n");
	printf("\tlongrunctl maxfrequency - Set maximum frequency mode\n");
}

//---------------------------------------------------------------
// Print longrun status
//---------------------------------------------------------------
void PrintStatus()
{
	int fd, moderesult, statresult;
	uint8 mode;
	uint  status[3];

	// Initialize variables
	fd = 0;
	mode = 0;
	moderesult = statresult = B_ERROR;
	status[0] = status[1] = status[2] = 0;
	
	// Open device
	fd = (int) open( LONGRUN_DEVICE_PATH, O_RDONLY);
	if( fd < 0)
	{
		printf("Failed to open Longrun driver!\n");
		return;
	}
	
	// Get current longrun mode
	moderesult = ioctl( fd, LONGRUN_CTL_GETMODE, (void*)&mode);
	if( moderesult == B_ERROR)
		printf("Failed to get longrun mode!\n");

	// Get current longrun status
	statresult = ioctl( fd, LONGRUN_CTL_GETSTATUS, (void*)status);
	if( statresult == B_ERROR)
		printf("Failed to get longrun status!\n");

	// Close device
	close(fd);

	// Print longrun mode to console
	if( mode > LONGRUN_MODE_MAX) mode = LONGRUN_MODE_MAX;
	printf("Mode:       %s\n", longrun_names[ mode] );
	printf("Frequency:  %i Mhz (%i%%)\n", status[0], status[2] );
	printf("Voltage:    %.3f\n", (float) status[1] / 1000.0f );

}

//---------------------------------------------------------------
// Set longrun mode
//---------------------------------------------------------------
void SetMode(uint8 mode)
{
	int fd, result;
	uint8 newmode;
	
	newmode = mode;
	
	
	// Validate mode
	if( newmode >= LONGRUN_MODE_MAX)
	{
		printf("SetMode: Unknown mode!\n");
		return;
	}

	
	// Open longrun device
	fd = (int) open( LONGRUN_DEVICE_PATH, O_RDONLY);
	if( fd < 0)
	{
		printf("Failed to open Longrun driver!\n");
		return;		
	}
	
	// Set new longrun mode
	result = ioctl( fd, LONGRUN_CTL_SETMODE, (void*)&newmode);	
	if( result == B_ERROR)
		printf("Error setting longrun mode!\n");
	
	// Close longrun device
	close( fd);
}

//---------------------------------------------------------------
//
// Entry point
//
//---------------------------------------------------------------
int main( int args, char *argv[] )
{
	int selection;

	selection = SELECTION_ERROR;

	// parse command line argument		
	if( args == 2)
	{
		if( strcmp( argv[1], "status") == 0)
			selection = SELECTION_STATUS;
			
		else if( strcmp( argv[1], "economy") == 0)
			selection = SELECTION_ECONOMY;
		
		else if( strcmp( argv[1], "performance") == 0)
			selection = SELECTION_PERFORMANCE;
			
		else if( strcmp( argv[1], "minfrequency") == 0)
			selection = SELECTION_MINFREQUENCY;
		
		else if( strcmp( argv[1], "maxfrequency") == 0)
			selection = SELECTION_MAXFREQUENCY;
	}
	
	
	// Carry out selection
	switch( selection)
	{
		// Print current status
		case SELECTION_STATUS:
			PrintStatus();
			break;
		// Set enonomy mode
		case SELECTION_ECONOMY:
			SetMode( LONGRUN_MODE_ECONOMY);
			break;
		// Set performance mode
		case SELECTION_PERFORMANCE:
			SetMode( LONGRUN_MODE_PERFORMANCE);
			break;
		// Set min frequency mode
		case SELECTION_MINFREQUENCY:
			SetMode( LONGRUN_MODE_MINFREQUENCY);
			break;
		// Set max frequency mode
		case SELECTION_MAXFREQUENCY:
			SetMode( LONGRUN_MODE_MAXFREQUENCY);
			break;
		// Invalid selection
		default:
			PrintInfo();
			break;
	}
	
	return 1;
}